#-*- coding: utf-8 -*-

# Copyright 2010 Calculate Ltd. http://www.calculate-linux.org
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
from cl_ldap_service import ldapService, __app__, __version__
from cl_opt import opt
import sys
from cl_share_cmd import share_cmd

# Перевод сообщений для программы
from cl_lang import lang
lang().setLanguage(sys.modules[__name__])

# Использование программы
USAGE = _("%prog [options]")

# Коментарии к использованию программы
COMMENT_EXAMPLES = _("Create LDAP tree")

# Пример использования программы
EXAMPLES = _("%prog")

# Описание программы (что делает программа)
DESCRIPTION = _("Creating LDAP tree")

# Опции командной строки
CMD_OPTIONS = [{'shortOption':"f",
                'longOption':"force",
                'help':_("forced setup service ldap")},
                {'longOption':"install",
                 'help':_("configure the system to install this package")},
                {'longOption':"uninstall",
                 'help':_("configure the system to uninstall this package")}]

class ldap_setup_cmd(share_cmd):
    def __init__(self):
        # Объект опций командной строки
        self.optobj = opt(package=__app__,
                          version=__version__,
                          usage=USAGE,
                          examples=EXAMPLES,
                          comment_examples=COMMENT_EXAMPLES,
                          description=DESCRIPTION,
                          option_list=CMD_OPTIONS +\
                                      opt.variable_control+opt.color_control,
                          check_values=self.checkOpts)
        # Создаем объект логики
        self.logicObj = ldapService()
        # Создаем переменные
        self.logicObj.createClVars()
        # Названия несовместимых опций
        self.optionsNamesIncompatible = ["install", "uninstall", "f"]

    def getIncompatibleOptions(self, optObj):
        """Получаем несовместимые опции"""
        retList = []
        for nameOpt in self.optionsNamesIncompatible:
            retList.append(getattr(optObj, nameOpt))
        return retList

    def _getNamesAllSetOptions(self):
        """Выдает словарь измененных опций"""
        setOptDict = self.optobj.values.__dict__.items()
        defaultOptDict = self.optobj.get_default_values().__dict__.items()
        return dict(set(setOptDict) - set(defaultOptDict)).keys()

    def getStringIncompatibleOptions(self):
        """Форматированная строка несовместимых опций разделенных ','"""
        listOpt = list(set(self.optionsNamesIncompatible) &\
                       set(self._getNamesAllSetOptions()))
        return ", ".join(map(lambda x: len(x) == 1 and "'-%s'"%x or "'--%s'"%x,\
                             listOpt))

    def checkOpts(self, optObj, args):
        """Check command line opt and arg"""
        if len(filter(lambda x: x, self.getIncompatibleOptions(optObj)))>1:
                errMsg = _("incompatible options")+":"+" %s"\
                            %self.getStringIncompatibleOptions()
                self.optobj.error(errMsg)
                return False
        if args:
            errMsg = _("incorrect argument") + ":" + " %s" %" ".join(args)
            self.optobj.error(errMsg)
            return False
        return optObj, args

    def setup(self, force=False):
        """Setup program"""
        return self.logicObj.setup(force=force)

    def install(self):
        """Инсталяция программы"""
        return self.logicObj.installProg()

    def uninstall(self):
        """Удаление программы"""
        return self.logicObj.uninstallProg()